@@ -327,12 +327,15 @@ class FileSystemStorage(ExternalStorage):
         while os.path.isdir(directory_path):
             try:
                 shutil.rmtree(directory_path)
-            except FileNotFoundError:
+            except (FileNotFoundError):
                 # If exception occurs when other IO workers are
                 # deleting the file at the same time.
                 pass
             except Exception:
-                logger.exception("Error cleaning up spill files")
+                logger.exception(
+                    "Error cleaning up spill files. "
+                    "You might still have remaining spilled "
+                    "objects inside `ray_spilled_objects` directory.")
                 break
 
 
