@@ -71,6 +71,7 @@ class ServeController:
         # Used to read/write checkpoints.
         self.controller_namespace = ray.get_runtime_context().namespace
         self.controller_name = controller_name
+        self.checkpoint_path = checkpoint_path
         kv_store_namespace = (
             f"{self.controller_name}-{self.controller_namespace}")
         self.kv_store = make_kv_store(
@@ -112,6 +113,10 @@ class ServeController:
         return self.deployment_state_manager._deployment_states[
             deployment_name]._replicas
 
+    def _stop_one_running_replica_for_testing(self, deployment_name):
+        self.deployment_state_manager._deployment_states[
+            deployment_name]._stop_one_running_replica_for_testing()
+
     async def wait_for_goal(self, goal_id: GoalId) -> Optional[Exception]:
         return await self.goal_manager.wait_for_goal(goal_id)
 
@@ -129,6 +134,9 @@ class ServeController:
         return await (
             self.long_poll_host.listen_for_change(keys_to_snapshot_ids))
 
+    def get_checkpoint_path(self) -> str:
+        return self.checkpoint_path
+
     def get_all_endpoints(self) -> Dict[EndpointTag, Dict[str, Any]]:
         """Returns a dictionary of deployment name to config."""
         return self.endpoint_state.get_endpoints()
