@@ -8,12 +8,12 @@ import platform
 import re
 import shutil
 import time
-from typing import Callable, Dict, Optional, Sequence, Union
+from typing import Dict, Optional, Sequence, Union
 import uuid
 
 import ray
 import ray.cloudpickle as cloudpickle
-from ray.exceptions import GetTimeoutError, RayActorError
+from ray.exceptions import RayActorError
 from ray.tune import TuneError
 from ray.tune.checkpoint_manager import Checkpoint, CheckpointManager
 # NOTE(rkn): We import ray.tune.registry here instead of importing the names we
@@ -21,20 +21,17 @@ from ray.tune.checkpoint_manager import Checkpoint, CheckpointManager
 # have been defined yet. See https://github.com/ray-project/ray/issues/1716.
 from ray.tune.registry import get_trainable_cls, validate_trainable
 from ray.tune.result import (DEFAULT_RESULTS_DIR, DONE, NODE_IP, PID,
-                             TRAINING_ITERATION, TRIAL_ID)
-from ray.tune.resources import Resources, \
-    json_to_resources, resources_to_json
+                             TRAINING_ITERATION, TRIAL_ID, DEBUG_METRICS)
+from ray.tune.resources import Resources
 from ray.tune.utils.placement_groups import PlacementGroupFactory, \
     resource_dict_to_pg_factory
 from ray.tune.utils.serialization import TuneFunctionEncoder
 from ray.tune.utils.trainable import TrainableUtil
 from ray.tune.utils import date_str, flatten_dict
-from ray.util import log_once
 from ray.util.annotations import DeveloperAPI
 from ray._private.utils import binary_to_hex, hex_to_binary
 
 DEBUG_PRINT_INTERVAL = 5
-CHECKPOINT_DELETER_NODE_IP_GET_TIMEOUT = 15
 logger = logging.getLogger(__name__)
 
 
@@ -86,26 +83,9 @@ class ExportFormat:
 class CheckpointDeleter:
     """Checkpoint deleter callback for a runner."""
 
-    def __init__(self,
-                 trial_id,
-                 runner,
-                 node_ip,
-                 timeout: int = CHECKPOINT_DELETER_NODE_IP_GET_TIMEOUT):
+    def __init__(self, trial_id, runner):
         self.trial_id = trial_id
         self.runner = runner
-        self.node_ip = node_ip
-        self._runner_ip = None
-        self.timeout = timeout
-
-    @property
-    def runner_ip(self):
-        if not self._runner_ip:
-            try:
-                self._runner_ip = ray.get(
-                    self.runner.get_current_ip.remote(), timeout=self.timeout)
-            except GetTimeoutError:
-                pass
-        return self._runner_ip
 
     def __call__(self, checkpoint):
         """Requests checkpoint deletion asynchronously.
@@ -117,22 +97,25 @@ class CheckpointDeleter:
             return
 
         if checkpoint.storage == Checkpoint.PERSISTENT and checkpoint.value:
-            logger.debug("Trial %s: Deleting checkpoint %s", self.trial_id,
-                         checkpoint.value)
             checkpoint_path = checkpoint.value
 
+            logger.debug("Trial %s: Deleting checkpoint %s", self.trial_id,
+                         checkpoint_path)
+
+            # TODO(ujvl): Batch remote deletes.
+            # We first delete the remote checkpoint. If it is on the same
+            # node as the driver, it will also remove the local copy.
+            ray.get(self.runner.delete_checkpoint.remote(checkpoint_path))
+
             # Delete local copy, if any exists.
-            if self.runner_ip != self.node_ip and os.path.exists(
-                    checkpoint_path):
+            if os.path.exists(checkpoint_path):
                 try:
                     checkpoint_dir = TrainableUtil.find_checkpoint_dir(
                         checkpoint_path)
                     shutil.rmtree(checkpoint_dir)
                 except FileNotFoundError:
-                    logger.warning("Checkpoint dir not found during deletion.")
-
-            # TODO(ujvl): Batch remote deletes.
-            self.runner.delete_checkpoint.remote(checkpoint.value)
+                    logger.debug(
+                        "Local checkpoint dir not found during deletion.")
 
 
 class TrialInfo:
@@ -148,10 +131,7 @@ class TrialInfo:
     def __init__(self, trial: "Trial"):
         self._trial_name = str(trial)
         self._trial_id = trial.trial_id
-        if trial.uses_placement_groups:
-            self._trial_resources = trial.placement_group_factory
-        else:
-            self._trial_resources = trial.resources
+        self._trial_resources = trial.placement_group_factory
 
     @property
     def trial_name(self):
@@ -184,6 +164,22 @@ def create_logdir(dirname, local_dir):
     return logdir
 
 
+def _to_pg_factory(resources: Optional[Resources],
+                   placement_group_factory: Optional[PlacementGroupFactory]
+                   ) -> PlacementGroupFactory:
+    """Outputs resources requirement in the form of PGF.
+
+    In case that `placement_group_factory` is None, `resources` will be
+    converted to PGF. If this is unsuccessful, an error will be raised.
+
+    """
+    if not placement_group_factory:
+        if not resources:
+            resources = Resources(cpu=1, gpu=0)
+        placement_group_factory = resource_dict_to_pg_factory(resources)
+    return placement_group_factory
+
+
 @DeveloperAPI
 class Trial:
     """A trial object holds the state for one model training run.
@@ -194,6 +190,10 @@ class Trial:
     Trials start in the PENDING state, and transition to RUNNING once started.
     On error it transitions to ERROR, otherwise TERMINATED on success.
 
+    There are resources allocated to each trial. It's preferred that resources
+    are specified using PlacementGroupFactory, rather than through Resources,
+    which is being deprecated.
+
     Attributes:
         trainable_name (str): Name of the trainable object to be executed.
         config (dict): Provided configuration dictionary with evaluated params.
@@ -201,8 +201,7 @@ class Trial:
         local_dir (str): Local_dir as passed to tune.run.
         logdir (str): Directory where the trial logs are saved.
         evaluated_params (dict): Evaluated parameters by search algorithm,
-        experiment_tag (str): Identifying trial name to show in the console.
-        resources (Resources): Amount of resources that this trial will use.
+        experiment_tag (str): Identifying trial name to show in the console
         status (str): One of PENDING, RUNNING, PAUSED, TERMINATED, ERROR/
         error_file (str): Path to the errors that this trial has raised.
 
@@ -213,6 +212,7 @@ class Trial:
         "best_result",
         "param_config",
         "extra_arg",
+        "placement_group_factory",
     ]
 
     PENDING = "PENDING"
@@ -232,7 +232,7 @@ class Trial:
                  placement_group_factory=None,
                  stopping_criterion=None,
                  remote_checkpoint_dir=None,
-                 sync_to_cloud=None,
+                 sync_function_tpl=None,
                  checkpoint_freq=0,
                  checkpoint_at_end=False,
                  sync_on_checkpoint=True,
@@ -243,20 +243,27 @@ class Trial:
                  trial_name_creator=None,
                  trial_dirname_creator=None,
                  log_to_file=None,
-                 max_failures=0):
+                 max_failures=0,
+                 stub=False):
         """Initialize a new trial.
 
         The args here take the same meaning as the command line flags defined
         in ray.tune.config_parser.
         """
-        validate_trainable(trainable_name)
+        # If this is set, trainables are not validated or looked up.
+        # This can be used e.g. to initialize Trial objects from checkpoints
+        # without loading the trainable first.
+        self.stub = stub
+
+        if not self.stub:
+            validate_trainable(trainable_name)
         # Trial config
         self.trainable_name = trainable_name
         self.trial_id = Trial.generate_id() if trial_id is None else trial_id
         self.config = config or {}
         self.local_dir = local_dir  # This remains unexpanded for syncing.
 
-        #: Parameters that Tune varies across searches.
+        # Parameters that Tune varies across searches.
         self.evaluated_params = evaluated_params or {}
         self.experiment_tag = experiment_tag
         trainable_cls = self.get_trainable_cls()
@@ -274,20 +281,16 @@ class Trial:
                         "clear the `resources_per_trial` option.".format(
                             trainable_cls, default_resources))
 
-                # New way: Trainable returns a PlacementGroupFactory object.
                 if isinstance(default_resources, PlacementGroupFactory):
                     placement_group_factory = default_resources
                     resources = None
-                # Set placement group factory to None for backwards
-                # compatibility.
                 else:
                     placement_group_factory = None
                     resources = default_resources
         self.location = Location()
 
-        self.resources = resources or Resources(cpu=1, gpu=0)
-        self.placement_group_factory = placement_group_factory
-        self._setup_resources()
+        self.placement_group_factory = _to_pg_factory(resources,
+                                                      placement_group_factory)
 
         self.stopping_criterion = stopping_criterion or {}
 
@@ -331,7 +334,12 @@ class Trial:
             self.remote_checkpoint_dir_prefix = remote_checkpoint_dir
         else:
             self.remote_checkpoint_dir_prefix = None
-        self.sync_to_cloud = sync_to_cloud
+
+        if sync_function_tpl == "auto" or not isinstance(
+                sync_function_tpl, str):
+            sync_function_tpl = None
+        self.sync_function_tpl = sync_function_tpl
+
         self.checkpoint_freq = checkpoint_freq
         self.checkpoint_at_end = checkpoint_at_end
         self.keep_checkpoints_num = keep_checkpoints_num
@@ -339,8 +347,7 @@ class Trial:
         self.sync_on_checkpoint = sync_on_checkpoint
         self.checkpoint_manager = CheckpointManager(
             keep_checkpoints_num, checkpoint_score_attr,
-            CheckpointDeleter(self._trainable_name(), self.runner,
-                              self.node_ip))
+            CheckpointDeleter(self._trainable_name(), self.runner))
 
         # Restoration fields
         self.restore_path = restore_path
@@ -366,37 +373,6 @@ class Trial:
         self._state_json = None
         self._state_valid = False
 
-    def _setup_resources(self, log_always: bool = False):
-        """Set up resource and placement group requirements.
-
-        This will try to convert the resource request in ``self.resources``
-        to a placement group factory object. If this is unsuccessful,
-        placement groups will not be used.
-
-        Args:
-            log_always (bool): If True, this will always log a warning if
-                conversion from a resource dict to a placement group
-                definition was unsuccessful (e.g. when passing ``extra_``
-                requests).
-
-
-        """
-        if not self.placement_group_factory and \
-           not int(os.getenv("TUNE_PLACEMENT_GROUP_AUTO_DISABLED", "0")):
-            try:
-                self.placement_group_factory = resource_dict_to_pg_factory(
-                    self.resources)
-            except ValueError as exc:
-                if log_always or log_once("tune_pg_extra_resources"):
-                    logger.warning(exc)
-                self.placement_group_factory = None
-
-        # Set placement group factory flag to True in Resources object.
-        if self.placement_group_factory:
-            resource_kwargs = self.resources._asdict()
-            resource_kwargs["has_placement_group"] = True
-            self.resources = Resources(**resource_kwargs)
-
     def _get_default_result_or_future(self) -> Optional[dict]:
         """Calls ray.get on self._default_result_or_future and assigns back.
 
@@ -468,6 +444,10 @@ class Trial:
 
     @property
     def remote_checkpoint_dir(self):
+        """This is the **per trial** remote checkpoint dir.
+
+        This is different from **per experiment** remote checkpoint dir.
+        """
         assert self.logdir, "Trial {}: logdir not initialized.".format(self)
         if not self.remote_checkpoint_dir_prefix:
             return None
@@ -475,8 +455,8 @@ class Trial:
         return os.path.join(self.remote_checkpoint_dir_prefix, logdir_name)
 
     @property
-    def uses_placement_groups(self):
-        return bool(self.placement_group_factory)
+    def uses_cloud_checkpointing(self):
+        return bool(self.remote_checkpoint_dir)
 
     def reset(self):
         # If there is `default_resource_request` associated with the trainable,
@@ -487,7 +467,6 @@ class Trial:
         trainable_cls = self.get_trainable_cls()
         clear_resources = (trainable_cls and
                            trainable_cls.default_resource_request(self.config))
-        resources = self.resources if not clear_resources else None
         placement_group_factory = (self.placement_group_factory
                                    if not clear_resources else None)
 
@@ -498,7 +477,7 @@ class Trial:
             local_dir=self.local_dir,
             evaluated_params=self.evaluated_params,
             experiment_tag=self.experiment_tag,
-            resources=resources,
+            resources=None,
             placement_group_factory=placement_group_factory,
             stopping_criterion=self.stopping_criterion,
             remote_checkpoint_dir=self.remote_checkpoint_dir,
@@ -523,8 +502,7 @@ class Trial:
             os.makedirs(self.logdir, exist_ok=True)
         self.invalidate_json_state()
 
-    def update_resources(
-            self, resources: Union[Dict, Callable, PlacementGroupFactory]):
+    def update_resources(self, resources: Union[Dict, PlacementGroupFactory]):
         """EXPERIMENTAL: Updates the resource requirements.
 
         Should only be called when the trial is not running.
@@ -535,12 +513,14 @@ class Trial:
         if self.status is Trial.RUNNING:
             raise ValueError("Cannot update resources while Trial is running.")
 
+        placement_group_factory = None
         if isinstance(resources, PlacementGroupFactory):
-            self.placement_group_factory = resources
+            placement_group_factory = resources
         else:
-            self.resources = Resources(**resources)
+            resources = Resources(**resources)
 
-        self._setup_resources()
+        self.placement_group_factory = _to_pg_factory(resources,
+                                                      placement_group_factory)
 
         self.invalidate_json_state()
 
@@ -554,15 +534,13 @@ class Trial:
             self._default_result_or_future = (
                 runner.get_auto_filled_metrics.remote(debug_metrics_only=True))
         self.checkpoint_manager.delete = CheckpointDeleter(
-            self._trainable_name(), runner, self.node_ip)
+            self._trainable_name(), runner)
         # No need to invalidate state cache: runner is not stored in json
         # self.invalidate_json_state()
 
     def set_location(self, location):
         """Sets the location of the trial."""
         self.location = location
-        if self.checkpoint_manager.delete:
-            self.checkpoint_manager.delete.node_ip = self.node_ip
         # No need to invalidate state cache: location is not stored in json
         # self.invalidate_json_state()
 
@@ -653,7 +631,7 @@ class Trial:
         """
         return self.num_failures < self.max_failures or self.max_failures < 0
 
-    def update_last_result(self, result, terminate=False):
+    def update_last_result(self, result):
         if self.experiment_tag:
             result.update(experiment_tag=self.experiment_tag)
 
@@ -661,7 +639,11 @@ class Trial:
         self.last_result = result
         self.last_update_time = time.time()
 
-        for metric, value in flatten_dict(result).items():
+        metric_result = self.last_result.copy()
+        for remove_metric in DEBUG_METRICS:
+            metric_result.pop(remove_metric, None)
+
+        for metric, value in flatten_dict(metric_result).items():
             if isinstance(value, Number):
                 if metric not in self.metric_analysis:
                     self.metric_analysis[metric] = {
@@ -697,6 +679,8 @@ class Trial:
         self.invalidate_json_state()
 
     def get_trainable_cls(self):
+        if self.stub:
+            return None
         return get_trainable_cls(self.trainable_name)
 
     def is_finished(self):
@@ -770,7 +754,6 @@ class Trial:
         Note this can only occur if the trial holds a PERSISTENT checkpoint.
         """
         state = self.__dict__.copy()
-        state["resources"] = resources_to_json(self.resources)
 
         for key in self._nonjson_fields:
             state[key] = binary_to_hex(cloudpickle.dumps(state.get(key)))
@@ -788,15 +771,19 @@ class Trial:
         return copy.deepcopy(state)
 
     def __setstate__(self, state):
-        state["resources"] = json_to_resources(state["resources"])
 
         if state["status"] == Trial.RUNNING:
             state["status"] = Trial.PENDING
         for key in self._nonjson_fields:
             state[key] = cloudpickle.loads(hex_to_binary(state[key]))
 
+        # Ensure that stub doesn't get overriden
+        stub = state.pop("stub", True)
         self.__dict__.update(state)
-        validate_trainable(self.trainable_name)
+        self.stub = stub or getattr(self, "stub", False)
+
+        if not self.stub:
+            validate_trainable(self.trainable_name)
 
         # Avoid creating logdir in client mode for returned trial results,
         # since the dir might not be creatable locally. TODO(ekl) thsi is kind
